# annotator class approach based feature exraction
# annotators are categorized into groups. Based on their group and output_level, features will be extreacted

# Classifiers are categorized into
# Classification_levels:
# 1. (One-To-One-Token)  (NER, POS, DEP)
# 2. (One-To-Some-Token)  (NER, POS, DEP)


#yake??
# 1. (Document/Sentence  to N?) (Yake)
# 2. (Document/Sentence to C class) (ClassifierDL,MultiClassifierDL/Sentiment/SentimentDl)

#1 to N token mappers
# Normalize, Stemm,

# Normalize maps 1 to N-K because it removes tokens and we get Nones